/*

	This file is part of GALE (Generic Adaptation Language and Engine).

    GALE is free software: you can redistribute it and/or modify it under the 
    terms of the GNU Lesser General Public License as published by the Free 
    Software Foundation, either version 3 of the License, or (at your option) 
    any later version.

    GALE is distributed in the hope that it will be useful, but WITHOUT ANY 
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
    FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for 
    more details.

    You should have received a copy of the GNU Lesser General Public License
    along with GALE. If not, see <http://www.gnu.org/licenses/>.
    
 */
/**
 * MCPlugin.java
 * Last modified: $Date$
 * In revision:   $Revision$
 * Modified by:   $Author: dsmits $
 *
 * Copyright (c) 2008-2011 Eindhoven University of Technology.
 * All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU LGPL license.
 */
package nl.tue.gale.ae.processor.plugin;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.RequestDispatcher;

import nl.tue.gale.ae.GaleContext;
import nl.tue.gale.ae.ProcessorException;
import nl.tue.gale.ae.Resource;
import nl.tue.gale.ae.processor.xmlmodule.MCModule;
import nl.tue.gale.common.GaleUtil;
import nl.tue.gale.common.cache.CacheSession;
import nl.tue.gale.common.code.Argument;
import nl.tue.gale.um.data.EntityValue;

import org.dom4j.DocumentFactory;
import org.dom4j.Element;

public class MCPlugin extends AbstractPlugin {
	private static final String xhtmlns = "http://www.w3.org/1999/xhtml";

	private boolean log = false;

	public boolean isLog() {
		return log;
	}

	public void setLog(boolean log) {
		this.log = log;
	}

	public void doGet(Resource resource) throws ProcessorException {
		doPost(resource);
	}

	@SuppressWarnings("unchecked")
	public void doPost(Resource resource) throws ProcessorException {
		GaleContext gale = GaleContext.of(resource);
		if ("true".equals(gale.req().getParameter("framewait")))
			return;
		try {
			DocumentFactory df = DocumentFactory.getInstance();
			Element html = df.createElement("html", xhtmlns);

			MCModule.Test test = null;
			try {
				test = (MCModule.Test) gale.req().getSession()
						.getAttribute(gale.req().getParameter("guid"));
			} catch (Exception e) {
			}
			if (test == null)
				throw new ProcessorException(
						"unable to find test-data in session");

			Element h1 = df.createElement("h1", xhtmlns).addText(
					test.getTitle());
			html.add(h1);
			Element ol = df.createElement("ol", xhtmlns);
			html.add(ol);

			int qnr = 0;
			double result = 0;
			for (MCModule.Question q : test.getQuestions()) {
				boolean correct = true;
				Element li = df.createElement("li", xhtmlns);
				li.add((Element) q.getQuestion().clone());
				ol.add(li);
				Element br = df.createElement("br", xhtmlns);
				li.add(br);
				for (MCModule.Answer a : q.getAnswers()) {
					int anr = q.getAnswers().indexOf(a);
					boolean checked = checkAnswer(qnr, anr, q.getRight() == 1,
							gale);
					correct = correct && (checked == a.isCorrect());
					Element ea = a.toXHTML(qnr, anr, q.getRight() == 1);
					ea.addAttribute("disabled", "true");
					if (checked)
						ea.addAttribute("checked", "true");
					li.add(ea);
					br = df.createElement("br", xhtmlns);
					li.add(br);
					if (checked && a.getExplain() != null
							&& !"".equals(a.getExplain())) {
						Element i = df.createElement("i", xhtmlns);
						li.add(i);
						i.add((Element) a.getExplain().clone());
						br = df.createElement("br", xhtmlns);
						li.add(br);
					}
				}
				br = df.createElement("br", xhtmlns);
				li.add(br);
				qnr++;
				result = (result / (double) qnr) * ((double) qnr - 1d);
				result += (correct ? (100d / (double) qnr) : 0d);
			}

			CacheSession<EntityValue> session = gale.openUmSession();
			gale.cm().execute(
					gale.cr(),
					test.getAction(),
					Argument.of("gale", "nl.tue.gale.ae.GaleContext", gale,
							"session", "nl.tue.gale.common.cache.CacheSession",
							session, "value", "java.lang.Double", new Double(
									result)));
			session.commit();

			if (isLog()) {
				// log the result
				StringBuffer sb = new StringBuffer();
				sb.append("\"");
				sb.append(gale.userId());
				sb.append("\";\"");
				sb.append(new Date());
				sb.append("\";\"");
				sb.append(gale.conceptUri());
				sb.append("\";");
				sb.append(result);
				sb.append("\n");
				gale.log().log("mctest", sb.toString());
			}

			if (test.isVerbose()) {
				if (test.getResult() != null)
					html.add((Element) test.getResult().clone());
				resource.put("xml", html);
				resource.put("mime", "text/xhtml");
				resource.put("original-url", "gale:/empty.xhtml");
				gale.usedStream();
			} else {
				try {
					Map<String, String[]> params = new HashMap<String, String[]>();
					params.putAll((Map<String, String[]>) gale.req()
							.getParameterMap());
					params.remove("plugin");
					params.remove("guid");
					gale.resp().sendRedirect(
							GaleUtil.getRequestURL(gale.req(), params));
				} catch (Exception e) {
					throw new ProcessorException("unable to redirect", e);
				}
				gale.usedResponse();
			}
		} catch (Exception e) {
			try {
				RequestDispatcher rd = gale.sc().getRequestDispatcher(
						"/ErrorServlet");
				gale.req()
						.getSession()
						.setAttribute(
								"exception",
								new ProcessorException(
										"unable to evaluate test", e));
				rd.forward(gale.req(), gale.resp());
				gale.usedResponse();
			} catch (Exception ee) {
				throw new ProcessorException(
						"unexpected error while trying to display the errorpage",
						ee);
			}
		}
	}

	private boolean checkAnswer(int qnr, int anr, boolean radio,
			GaleContext gale) {
		if (radio)
			return ("a_" + anr).equals(gale.req().getParameter("q_" + qnr));
		else
			return ("true").equals(gale.req().getParameter(
					"q_" + qnr + "_" + anr));
	}
}